#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: xxlin
@Date: 2019-04-10 13:27:58
@LastEditTime: 2019-04-14 10:36:27
'''

class COLOR:
    black = 30  #  黑色
    red = 31  #  红色
    green = 32  #  绿色
    yellow = 33  #  黄色
    blue = 34  #  蓝色
    purple = 35  #  紫红色
    cyan = 36  #  青蓝色
    white = 37  #  白色
    
class BRUTER_RESULT_STATUS:
    FAIL = 0
    SUCCESS = 1
    RETRAY = 2
